<?php

namespace BeycanPress\CryptoPay\Donate;

class Loader extends PluginHero\Plugin
{
    public function __construct($pluginFile)
    {
        parent::__construct([
            'pluginFile' => $pluginFile,
            'textDomain' => 'cryptopay',
            'pluginKey' => 'cryptopay_donate',
            'settingKey' => 'cryptopay_donate_settings',
            'pluginVersion' => '1.3.0'
        ]);
        
        if ($this->setting('license')) {
            if ($this->setting('payment_address') == '') {
                $this->adminNotice(esc_html__('If you did not specify a payment address in the CryptoPay Donate settings, the plugin will not work. Please specify a payment address first.', 'cryptopay'), 'error');
            } else {
                if (!is_admin()) {
                    new DonateBox\DonateBox();
                }
                
                new Integrations();
            }
        } else {
            $this->adminNotice(esc_html__('In order to use the "CryptoPay Donate" Plugin, please enter your license (purchase) code in the license field in the settings section.', 'cryptopay'), 'error');
        }
    }

    public function adminProcess()
    {
        (new Models\Address())->createTable();
        (new Models\Transaction())->createTable();
        
        new Pages\Donations();
        new Pages\AddressMapping();
        
        if (in_array('walletconnect', Settings::getAcceptedWallets()) && $this->setting('infuraProjectId') == '') {
            $this->adminNotice(esc_html__('Please enter an infura project id for WalletConnect to work. - CryptoPay Donate', 'cryptopay'), 'error');
        }
        
        add_action('init', function(){
            new Settings;
        }, 9);
    }

    public static function activation()
    {
        (new Models\Address())->createTable();
        (new Models\Transaction())->createTable();
    }

    public static function uninstall()
    {
        $settings = get_option(self::$instance->settingKey);
        if (isset($settings['dds']) && $settings['dds']) {
            delete_option(self::$instance->settingKey);
            delete_option(strtolower('widget_'.DonateBox\DonateBoxWidget::class));
            (new Models\Address())->drop();
            (new Models\Transaction())->drop();
        }
    }
}
