<?php 

namespace BeycanPress\CryptoPay\Donate\Pages;

use \Beycan\WPTable\Table;
use \BeycanPress\CryptoPay\Donate\PluginHero\Page;
use \BeycanPress\CryptoPay\Donate\Models\Address;

class AddressMapping extends Page
{   
    /**
     * Class construct
     * @return void
     */
    public function __construct()
    {
        parent::__construct([
            'pageName' => esc_html__('Address mapping', 'cryptopay'),
            'parent' => $this->pages->Donations->slug
        ]);
    }

    /**
     * @param string $key
     * @return bool
     */
    private function isAddress(string $address) : bool
    {
        if (!is_string($address)) {
            return false;
        }
        return (preg_match('/^0x[a-fA-F0-9]{40}$/', $address) >= 1);
    }

    /**
     * @return void
     */
    public function page()
    {
        $address = new Address();

        if (isset($_GET['id']) && $address->delete(['id' => absint($_GET['id'])])) {
            $this->notice(esc_html__('Successfully deleted!', 'cryptopay'), 'success', true);
        }
        
        if (isset($_POST['user_id'])) {
            $userId = sanitize_text_field($_POST['user_id']);
            $address1 = sanitize_text_field($_POST['address']);

            if (!$userId || !$address1) {
                $this->notice(esc_html__('Please fill in all fields!', 'cryptopay'), 'error', true);
            }

            if (!$this->isAddress($address1)) {
                $this->notice(esc_html__('Invalid receiver address!', 'cryptopay'), 'error', true);
            }

            if ($userId && $address1 && $this->isAddress($address1)) {

                $alreadyMatch = $address->findBy([
                    'address' => $address1
                ]);

                if ($alreadyMatch) {
                    $this->notice(esc_html__('This address is already match!', 'cryptopay'), 'error', true);
                } else {
                    $result = $address->insert([
                        'user_id' => $userId,
                        'address' => $address1
                    ]);
        
                    if ($result) {
                        $this->notice(esc_html__('Matching successful', 'cryptopay'), 'success', true);
                    } else {
                        $this->notice(esc_html__('There was a problem matching!', 'cryptopay'), 'error', true);
                    }
                }
            }
        }

        $table = (new Table($address))->setColumns([
                'user_id' => esc_html__('User id', 'cryptopay'),
                'address' => esc_html__('Address', 'cryptopay'),
                'delete'  => esc_html__('Delete', 'cryptopay')
        ])->setOptions([
            'search' => [
                'id' => 'search-box',
                'title' => esc_html__('Search...', 'cryptopay')
            ]
        ])
        ->setOrderQuery(['created_at', 'desc'])
        ->addHooks([
            'delete' => function($address) {
                return '<a class="button" href="'.$this->getCurrentUrl() . '&id=' . $address->id.'">'.esc_html__('Delete', 'cryptopay').'</a>';
            }
        ])
        ->setSortableColumns([
            'created_at'
        ])
        ->createDataList();

        $this->viewEcho('pages/address-mapping/index', [
            'table' => $table
        ]);
    }

}