<?php 

namespace BeycanPress\CryptoPay\Donate\Pages;

use \Beycan\WPTable\Table;
use \BeycanPress\CryptoPay\Donate\PluginHero\Page;
use \BeycanPress\CryptoPay\Donate\Models\Transaction;

class Donations extends Page
{   
    /**
     * Class construct
     * @return void
     */
    public function __construct()
    {
        parent::__construct([
            'pageName' => esc_html__('CryptoPay Donate', 'cryptopay'),
            'subMenuPageName' => esc_html__('Donations', 'cryptopay'),
            'icon' => $this->getImageUrl('menu.png'),
            'subMenu' => true
        ]);
    }

    /**
     * @return void
     */
    public function page()
    {
        $transaction = new Transaction();

        if (isset($_GET['id']) && $transaction->delete(['id' => absint($_GET['id'])])) {
            $this->notice(esc_html__('Successfully deleted!', 'cryptopay'), 'success', true);
        }

        $table = (new Table($transaction))
        ->setColumns([
            'transaction_id'   => esc_html__('Transaction id', 'cryptopay'),
            'chain_id'         => esc_html__('Chain id', 'cryptopay'),
            'chain_name'       => esc_html__('Chain name', 'cryptopay'),
            'used_wallet'      => esc_html__('Used wallet', 'cryptopay'),
            'donate_amount'    => esc_html__('Donate amount', 'cryptopay'),
            'receiver'         => esc_html__('Receiver address', 'cryptopay'),
            'sender_address'   => esc_html__('Sender address', 'cryptopay'),
            'created_at'       => esc_html__('Created at', 'cryptopay'),
            'delete'           => esc_html__('Delete', 'cryptopay')
        ])
        ->setOptions([
            'search' => [
                'id' => 'search-box',
                'title' => esc_html__('Search...', 'cryptopay')
            ]
        ])
        ->setOrderQuery(['created_at', 'desc'])
        ->addHooks([
            'transaction_id' => function($transaction) {
                $paymentInfo = unserialize($transaction->payment_info);
                $explorerUrl = rtrim($paymentInfo->usedChain->explorerUrl, '/');
                $url = $explorerUrl.'/tx/'.$transaction->transaction_id;
                return '<a href="'.esc_url($url).'" target="_blank">'.esc_html($transaction->transaction_id).'</a>';
            },
            'chain_id' => function($transaction) {
                $paymentInfo = unserialize($transaction->payment_info);
                return esc_html($paymentInfo->usedChain->id);
            },
            'chain_name' => function($transaction) {
                $paymentInfo = unserialize($transaction->payment_info);
                return esc_html($paymentInfo->usedChain->name);
            },
            'used_wallet' => function($transaction) {
                $paymentInfo = unserialize($transaction->payment_info);
                return isset($paymentInfo->usedWallet) ? esc_html($paymentInfo->usedWallet) : null;
            },
            'donate_amount' => function($transaction) {
                $paymentInfo = unserialize($transaction->payment_info);
                return esc_html($paymentInfo->paymentPrice . " " . $paymentInfo->paymentCurrency);
            },
            'sender_address' => function($transaction) {
                $paymentInfo = unserialize($transaction->payment_info);
                $explorerUrl = rtrim($paymentInfo->usedChain->explorerUrl, '/');
                $url = $explorerUrl.'/address/'.$paymentInfo->senderAddress;
                return '<a href="'.esc_url($url).'" target="_blank">'.esc_html($paymentInfo->senderAddress).'</a>';
            },
            'receiver' => function($transaction) {
                $paymentInfo = unserialize($transaction->payment_info);
                $explorerUrl = rtrim($paymentInfo->usedChain->explorerUrl, '/');
                $url = $explorerUrl.'/address/'.$transaction->receiver;
                return '<a href="'.esc_url($url).'" target="_blank">'.esc_html($transaction->receiver).'</a>';
            },
            'delete' => function($transaction) {
                return '<a class="button" href="'.$this->getCurrentUrl() . '&id=' . $transaction->id.'">'.esc_html__('Delete', 'cryptopay').'</a>';
            }
        ])
        ->setSortableColumns(['created_at'])
        ->createDataList(function(object $model, array $orderQuery, int $limit, int $offset) {
            if (isset($_GET['status']) && !empty($_GET['status'])) {
                $s = sanitize_text_field($_GET['status']);
                if (isset($dataList)) {
                    $dataList = array_filter($dataList, function($obj) use ($s) {
                        return $obj->status == $s;
                    });
                    $dataListCount = count($dataList);
                } else {
                    $dataList = $model->findBy([
                        'status' => $s,
                    ], $orderQuery, $limit, $offset);
                    $dataListCount = $model->getCount([
                        'status' => $s,
                    ]);
                }

                return [$dataList, $dataListCount];
            } 

            return null;
        });

        $this->viewEcho('pages/donations/index', [
            'table' => $table
        ]);
    }

}