<?php

namespace BeycanPress\CryptoPay\Donate;

use \BeycanPress\CryptoPay\Donate\PluginHero\Plugin;
use \BeycanPress\CryptoPay\Donate\PluginHero\Setting;

class Settings extends Setting
{
    use PluginHero\Helpers;

    public function __construct()
    {
        $parent = $this->pages->Donations->slug;
        parent::__construct(esc_html__('Settings', 'cryptopay'), $parent);
        $this->licensed();

        self::createSection(array(
            'id'     => 'general_options', 
            'title'  => esc_html__('General options', 'cryptopay'),
            'icon'   => 'fa fa-cog',
            'fields' => array(
                array(
                    'id'      => 'dds',
                    'title'   => esc_html__('Data deletion status', 'cryptopay'),
                    'type'    => 'switcher',
                    'default' => false,
                    'help'    => esc_html__('This setting is passive come by default. You enable this setting. All data created by the plug-in will be deleted while removing the plug-in.', 'cryptopay')
                ),
                array(
                    'id'      => 'shortCode',
                    'title'   => esc_html__('Shorcode', 'cryptopay'),
                    'type'    => 'content',
                    'content' => '[cryptopay-donate-box]'
                ),
                array(
                    'id'      => 'payment_address',
                    'title'   => esc_html__('Payment address', 'cryptopay'),
                    'type'    => 'text',
                    'help'    => esc_html__('The account address to which the payments will be transferred. (BEP20, ERC20, MetaMask, Trust Wallet, Binance Wallet )', 'cryptopay'),
                    'sanitize' => function($val) {
						return sanitize_text_field($val);
					},
                    'validate' => function($val) {
                        $val = sanitize_text_field($val);
                        if (empty($val)) {
                            return esc_html__('Payment address cannot be empty.', 'cryptopay');
                        } elseif (strlen($val) < 42 || strlen($val) > 42) {
                            return esc_html__('Payment address must consist of 42 characters.', 'cryptopay');
                        }
                    }
                ),
                array(
                    'id'      => 'show_in_posts',
                    'title'   => esc_html__('Show in posts', 'cryptopay'),
                    'type'    => 'select',
                    'help'    => esc_html__('It allows you to add a donation box at the beginning or end of all the posts you share.', 'cryptopay'),
                    'options' => [
                        'not-show'      => esc_html__('Not show', 'cryptopay'),
                        'show-in-begin' => esc_html__('Show in begin', 'cryptopay'),
                        'show-in-end'   => esc_html__('Show in end', 'cryptopay')
                    ],
                    'default' => 'not-show'
                ),
                array(
                    'id'      => 'show_in_pages',
                    'title'   => esc_html__('Show in pages', 'cryptopay'),
                    'type'    => 'select',
                    'help'    => esc_html__('It allows you to add a donation box at the beginning or end of all the pages you share.', 'cryptopay'),
                    'options' => [
                        'not-show'      => esc_html__('Not show', 'cryptopay'),
                        'show-in-begin' => esc_html__('Show in begin', 'cryptopay'),
                        'show-in-end'   => esc_html__('Show in end', 'cryptopay')
                    ],
                    'default' => 'not-show'
                ),
                array(
                    'id'      => 'testnets',
                    'title'   => esc_html__('Testnets', 'cryptopay'),
                    'type'    => 'switcher',
                    'help'    => esc_html__('When you activate this setting, predefined testnets are activated.', 'cryptopay'),
                    'default' => false,
                ),
            )
        ));

        self::createSection(array(
            'id'     => 'wallets_menu', 
            'title'  => esc_html__('Accepted wallets', 'cryptopay'),
            'icon'   => 'fa fa-wallet',
            'fields' => array(
                array(
                    'id'     => 'acceptedWallets',
                    'type'   => 'fieldset',
                    'title'  => esc_html__('Wallets', 'cryptopay'),
                    'help'   => esc_html__('Specify the wallets you want to accept payments from.', 'cryptopay'),
                    'fields' => array(
                        array(
                            'id'      => 'metamask',
                            'title'   => esc_html('MetaMask'),
                            'type'    => 'switcher',
                            'default' => true,
                        ),
                        array(
                            'id'      => 'trustwallet',
                            'title'   => esc_html('Trust Wallet'),
                            'type'    => 'switcher',
                            'default' => true,
                        ),
                        array(
                            'id'      => 'binancewallet',
                            'title'   => esc_html('Binance Wallet'),
                            'type'    => 'switcher',
                            'default' => true,
                        ),
                        array(
                            'id'      => 'walletconnect',
                            'title'   => esc_html('WalletConnect'),
                            'type'    => 'switcher',
                            'default' => true,
                        ),
                    ),
                    'validate' => function($val) {
                        foreach ($val as $value) {
                            if ($value) {
                                break;
                            } else {
                                return esc_html__('You must activate at least one wallet!', 'cryptopay');
                            }
                        }
                    }
                ),
            ) 
        ));

        self::createSection(array(
            'id'     => 'networks', 
            'title'  => esc_html__('Accepted networks', 'cryptopay'),
            'icon'   => 'fa fa-link',
            'fields' => array(
                array(
                    'id'      => 'accepted_chains',
                    'title'   => esc_html__('Accepted networks', 'cryptopay'),
                    'type'    => 'group',
                    'help'    => esc_html__('Add the blockchain networks you accept to receive payments.', 'cryptopay'),
                    'button_title' => esc_html__('Add new', 'cryptopay'),
                    'default' => [
                        [
                            'name' =>  'Main Ethereum Network',
                            'rpc_url' =>  'https://mainnet.infura.io/v3/9aa3d95b3bc440fa88ea12eaa4456161',
                            'id' =>  1,
                            'explorer_url' =>  'https://etherscan.io/',
                            'active' => true,
                            'native_currency' => [
                                'active' =>  true,
                                'symbol' =>  'ETH',
                                'decimals' =>  18,
                                'image' => $this->getImageUrl('eth.png'),
                            ],
                            'currencies' => [
                                [ 
                                    'symbol' =>  'USDT',
                                    'address' =>  '0xdac17f958d2ee523a2206206994597c13d831ec7',
                                    'image' =>  $this->getImageUrl('usdt.png'),
                                    'active' => true
                                ],
                                [ 
                                    'symbol' =>  'USDC',
                                    'address' =>  '0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48',
                                    'image' =>  $this->getImageUrl('usdc.png'),
                                    'active' => true
                                ],
                                [ 
                                    'symbol' =>  'BUSD',
                                    'address' =>  '0x4Fabb145d64652a948d72533023f6E7A623C7C53',
                                    'image' =>  $this->getImageUrl('busd.png'),
                                    'active' => true
                                ],
                                [ 
                                    'symbol' =>  'DAI',
                                    'address' =>  '0x6b175474e89094c44da98b954eedeac495271d0f',
                                    'image' =>  $this->getImageUrl('dai.png'),
                                    'active' => true
                                ],
                                [ 
                                    'symbol' =>  'BNB',
                                    'address' =>  '0xB8c77482e45F1F44dE1745F52C74426C631bDD52',
                                    'image' =>  $this->getImageUrl('bnb.png'),
                                    'active' => false
                                ],
                            ]
                        ],
                        [
                            'name' =>  'Binance Smart Chain',
                            'rpc_url' =>  'https://bsc-dataseed.binance.org/',
                            'id' =>  56,
                            'explorer_url' =>  'https://bscscan.com/',
                            'active' => true,
                            'native_currency' => [
                                'active' =>  true,
                                'symbol' =>  'BNB',
                                'decimals' =>  18,
                                'image' => $this->getImageUrl('bnb.png'),
                            ],
                            'currencies' => [
                                [ 
                                    'symbol' =>  'BUSD',
                                    'address' =>  '0xe9e7cea3dedca5984780bafc599bd69add087d56',
                                    'image' =>  $this->getImageUrl('busd.png'),
                                    'active' => true
                                ],
                                [
                                    'symbol' =>  'USDT',
                                    'address' =>  '0x55d398326f99059ff775485246999027b3197955',
                                    'image' =>  $this->getImageUrl('usdt.png'),
                                    'active' => true
                                ],
                                [
                                    'symbol' =>  'USDC',
                                    'address' =>  '0x8ac76a51cc950d9822d68b83fe1ad97b32cd580d',
                                    'image' =>  $this->getImageUrl('usdc.png'),
                                    'active' => true
                                ],
                                [
                                    'symbol' =>  'DAI',
                                    'address' =>  '0x1af3f329e8be154074d8769d1ffa4ee058b1dbc3',
                                    'image' =>  $this->getImageUrl('dai.png'),
                                    'active' => true
                                ],
                                [
                                    'symbol' =>  'ETH',
                                    'address' =>  '0x2170ed0880ac9a755fd29b2688956bd959f933f8',
                                    'image' =>  $this->getImageUrl('eth.png'),
                                    'active' => false
                                ],
                                [
                                    'symbol' =>  'LTC',
                                    'address' =>  '0x4338665cbb7b2485a8855a139b75d5e34ab0db94',
                                    'image' =>  $this->getImageUrl('ltc.png'),
                                    'active' => false
                                ],
                                [
                                    'symbol' =>  'DOGE',
                                    'address' =>  '0xba2ae424d960c26247dd6c32edc70b295c744c43',
                                    'image' =>  $this->getImageUrl('doge.png'),
                                    'active' => false
                                ]
                            ]
                        ],
                        [
                            'name' =>  'Avalanche Network',
                            'rpc_url' =>  'https://api.avax.network/ext/bc/C/rpc',
                            'id' =>  43114,
                            'explorer_url' =>  'https://cchain.explorer.avax.network/',
                            'active' => true,
                            'native_currency' => [
                                'active' =>  true,
                                'symbol' =>  'AVAX',
                                'decimals' =>  18,
                                'image' => $this->getImageUrl('avax.png'),
                            ],
                            'currencies' => [
                                [ 
                                    'symbol' =>  'USDT',
                                    'address' =>  '0xde3a24028580884448a5397872046a019649b084',
                                    'image' =>  $this->getImageUrl('usdt.png'),
                                    'active' => true
                                ],
                                [ 
                                    'symbol' =>  'DAI',
                                    'address' =>  '0xba7deebbfc5fa1100fb055a87773e1e99cd3507a',
                                    'image' =>  $this->getImageUrl('dai.png'),
                                    'active' => true
                                ],
                                [ 
                                    'symbol' =>  'ETH',
                                    'address' =>  '0xf20d962a6c8f70c731bd838a3a388D7d48fA6e15',
                                    'image' =>  $this->getImageUrl('eth.png'),
                                    'active' => true
                                ],
                            ]
                        ],
                        [
                            'name' =>  'Polygon Mainnet',
                            'rpc_url' =>  'https://rpc-mainnet.matic.network',
                            'id' =>  137,
                            'explorer_url' =>  'https://polygonscan.com/',
                            'active' => true,
                            'native_currency' => [
                                'active' =>  true,
                                'symbol' =>  'MATIC',
                                'decimals' => 18,
                                'image' => $this->getImageUrl('matic.png'),
                            ],
                            'currencies' => [
                                [ 
                                    'symbol' =>  'USDT',
                                    'address' =>  '0xc2132d05d31c914a87c6611c10748aeb04b58e8f',
                                    'image' =>  $this->getImageUrl('usdt.png'),
                                    'active' => true
                                ],
                                [ 
                                    'symbol' =>  'USDC',
                                    'address' =>  '0x2791bca1f2de4661ed88a30c99a7a9449aa84174',
                                    'image' =>  $this->getImageUrl('usdc.png'),
                                    'active' => true
                                ],
                                [ 
                                    'symbol' =>  'DAI',
                                    'address' =>  '0x8f3Cf7ad23Cd3CaDbD9735AFf958023239c6A063',
                                    'image' =>  $this->getImageUrl('dai.png'),
                                    'active' => true
                                ],
                            ]
                        ]
                    ],
                    'sanitize' => function($val) {
                        if (is_array($val)) {
                            foreach ($val as $key => &$value) {
                                $value['name'] = sanitize_text_field($value['name']);
                                $value['rpc_url'] = sanitize_text_field($value['rpc_url']);
                                $value['id'] = absint($value['id']);
                                $value['explorer_url'] = sanitize_text_field($value['explorer_url']);
                                $value['native_currency']['symbol'] = strtoupper(sanitize_text_field($value['native_currency']['symbol']));
                                $value['native_currency']['decimals'] = absint($value['native_currency']['decimals']);
                                $value['native_currency']['image'] = sanitize_text_field($value['native_currency']['image']);
                                if (isset($value['currencies'])) {
                                    foreach ($value['currencies'] as $key => &$currency) {
                                        $currency['symbol'] = strtoupper(sanitize_text_field($currency['symbol']));
                                        $currency['address'] = sanitize_text_field($currency['address']);
                                        $currency['image'] = sanitize_text_field($currency['image']); 
                                    }
                                }
                            }
                        }

                        return $val;
                    },
                    'validate' => function($val) {
                        if (is_array($val)) {
                            foreach ($val as $key => $value) {
                                if (empty($value['name'])) {
                                    return esc_html__('Network name cannot be empty.', 'cryptopay');
                                } elseif (empty($value['rpc_url'])) {
                                    return esc_html__('Network RPC URL cannot be empty.', 'cryptopay');
                                } elseif (empty($value['id'])) {
                                    return esc_html__('Chain ID cannot be empty.', 'cryptopay');
                                } elseif (empty($value['explorer_url'])) {
                                    return esc_html__('Explorer URL cannot be empty.', 'cryptopay');
                                } elseif (empty($value['native_currency']['symbol'])) {
                                    return esc_html__('Native currency symbol cannot be empty.', 'cryptopay');
                                } elseif (empty($value['native_currency']['decimals'])) {
                                    return esc_html__('Native currency Decimals cannot be empty.', 'cryptopay');
                                } elseif (empty($value['native_currency']['image'])) {
                                    return esc_html__('Native currency Image cannot be empty.', 'cryptopay');
                                } elseif (!isset($value['currencies'])) {
                                    return esc_html__('You must add at least one currency!', 'cryptopay');
                                } elseif (isset($value['currencies'])) {
                                    foreach ($value['currencies'] as $key => $currency) {
                                        if (empty($currency['symbol'])) {
                                            return esc_html__('Currency symbol cannot be empty.', 'cryptopay');
                                        } elseif (empty($currency['address'])) {
                                            return esc_html__('Currency contract address cannot be empty.', 'cryptopay');
                                        } elseif (strlen($currency['address']) < 42 || strlen($currency['address']) > 42) {
                                            return esc_html__('Currency contract address must consist of 42 characters.', 'cryptopay');
                                        } elseif (empty($currency['image'])) {
                                            return esc_html__('Currency image cannot be empty.', 'cryptopay');
                                        }  
                                    }
                                }
                            }
                        } else {
                            return esc_html__('You must add at least one blockchain network!', 'cryptopay');
                        }
                    },
                    'fields'    => array(
                        array(
                            'title' => esc_html__('Network name', 'cryptopay'),
                            'id'    => 'name',
                            'type'  => 'text'
                        ),
                        array(
                            'title' => esc_html__('Network RPC URL', 'cryptopay'),
                            'id'    => 'rpc_url',
                            'type'  => 'text',
                        ),
                        array(
                            'title' => esc_html__('Chain ID', 'cryptopay'),
                            'id'    => 'id',
                            'type'  => 'number'
                        ),
                        array(
                            'title' => esc_html__('Explorer URL', 'cryptopay'),
                            'id'    => 'explorer_url',
                            'type'  => 'text'
                        ),
                        array(
                            'id'      => 'active',
                            'title'   => esc_html__('Active/Passive', 'cryptopay'),
                            'type'    => 'switcher',
                            'help'    => esc_html__('Get paid in this network?', 'cryptopay'),
                            'default' => true,
                        ),
                        array(
                            'id'     => 'native_currency',
                            'type'   => 'fieldset',
                            'title'  => esc_html__('Native currency', 'cryptopay'),
                            'fields' => array(
                                array(
                                    'id'      => 'active',
                                    'title'   => esc_html__('Active/Passive', 'cryptopay'),
                                    'type'    => 'switcher',
                                    'help'    => esc_html__('Get paid in native currency?', 'cryptopay'),
                                    'default' => true,
                                ),
                                array(
                                    'id'    => 'symbol',
                                    'type'  => 'text',
                                    'title' => esc_html__('Symbol', 'cryptopay')
                                ),
                                array(
                                    'id'    => 'decimals',
                                    'type'  => 'number',
                                    'title' => esc_html__('Decimals', 'cryptopay')
                                ),
                                array(
                                    'title' => esc_html__('Image', 'cryptopay'),
                                    'id'    => 'image',
                                    'type'  => 'upload'
                                ),
                            ),
                        ),
                        array(
                            'id'        => 'currencies',
                            'type'      => 'group',
                            'title'     => esc_html__('Currencies', 'cryptopay'),
                            'button_title' => esc_html__('Add new', 'cryptopay'),
                            'fields'    => array(
                                array(
                                    'title' => esc_html__('Symbol', 'cryptopay'),
                                    'id'    => 'symbol',
                                    'type'  => 'text'
                                ),
                                array(
                                    'title' => esc_html__('Contract address', 'cryptopay'),
                                    'id'    => 'address',
                                    'type'  => 'text'
                                ),
                                array(
                                    'title' => esc_html__('Image', 'cryptopay'),
                                    'id'    => 'image',
                                    'type'  => 'upload'
                                ),
                                array(
                                    'id'      => 'active',
                                    'title'   => esc_html__('Active/Passive', 'cryptopay'),
                                    'type'    => 'switcher',
                                    'help'    => esc_html__('You can easily activate or deactivate Token without deleting it.', 'cryptopay'),
                                    'default' => true,
                                ),
                            ),
                        ),
                    ),
                ),
            ) 
        ));

        self::createSection(array(
            'id'     => 'apis', 
            'title'  => esc_html__('API\'s', 'cryptopay'),
            'icon'   => 'fa fa-project-diagram',
            'fields' => array(
                array(
                    'id' => 'infuraProjectId',
                    'type'  => 'text',
                    'title' => esc_html__('Infura Project ID', 'cryptopay'),
                    'help'  => esc_html__('Please enter an infura project id for WalletConnect to work.', 'cryptopay'),
                    'sanitize' => function($val) {
                        return sanitize_text_field($val);
                    }
                ),
            ) 
        ));

        self::createSection(array(
            'id'     => 'backup', 
            'title'  => esc_html__('Backup', 'cryptopay'),
            'icon'   => 'fa fa-shield',
            'fields' => array(
                array(
                    'type'  => 'backup',
                    'title' => esc_html__('Backup', 'cryptopay')
                ),
            ) 
        ));
    }

    public static function getAcceptedWallets() : array
    {
		$acceptedWallets = Plugin::$instance->setting('acceptedWallets');
		
		if (!$acceptedWallets) return [];
		
        $acceptedWallets = array_filter($acceptedWallets, function($val) {
            return $val;
        });

        return array_keys($acceptedWallets);
    }
}